@@ -28,7 +28,7 @@
 
 
 mxMI <- function(model, matrices=NA, full=TRUE){
-  warnModelCreatedByOldVersion(model)
+	warnModelCreatedByOldVersion(model)
 	if(single.na(matrices)){
 		matrices <- names(model$matrices) #names of them rather
 		if (is(model$expectation, "MxExpectationRAM")) {
@@ -106,12 +106,12 @@ mxMI <- function(model, matrices=NA, full=TRUE){
 				# Create and run the single-parameter model for the LISREL-type/partial/[lower bound] MI
 				gmodel <- mxModel(gmodel, custom.compute)
 				grun <- try(mxRun(gmodel, silent = FALSE, suppressWarnings = FALSE, unsafe=TRUE)) #suppress Warnings =TRUE
-        if (is(grun, "try-error")) {
-          # oops, not happy about that parameter (e.g., diag of RAM's A)
-          gmodel <- omxSetParameters(gmodel, labels=names(omxGetParameters(gmodel)), free=FALSE)
-          next
-        }
-
+				if (is(grun, "try-error")) {
+					# oops, not happy about that parameter (e.g., diag of RAM's A)
+					gmodel <- omxSetParameters(gmodel, labels=names(omxGetParameters(gmodel)), free=FALSE)
+					next
+				}
+				
 				# restricted MI
 				grad <- grun$output$gradient #get gradient
 				hess <- grun$output$hessian #get Hessian
@@ -126,9 +126,14 @@ mxMI <- function(model, matrices=NA, full=TRUE){
 					
 					# full MI
 					grad.full <- plusOneParamRun$output$gradient
-					grad.full[is.na(grad.full)] <- 0
+					#grad.full[is.na(grad.full)] <- 0
+					grad.full[which(names(grad.full)!=names(omxGetParameters(gmodel)))] <- 0
 					hess.full <- plusOneParamRun$output$hessian
-					modind.full <- 0.5*t(matrix(grad.full)) %*% solve(hess.full) %*% matrix(grad.full)
+					solve_hess_full <- try(solve(hess.full),silent=F)
+					if(is(solve_hess_full,"try-error") && length(grep("computationally singular",solve_hess_full[1]))){
+						solve_hess_full <- chol2inv(chol(hess.full))
+					}
+					modind.full <- 0.5*t(matrix(grad.full)) %*% solve_hess_full %*% matrix(grad.full)
 					if(sum(grad.full != 0) == 1){
 						exppar.full <- - modind.full/grad.full[grad.full != 0]
 					} else {stop("Something strange in the neighborhood.\nFound a one-parameter model with more than one parameter.\nPost this to the OpenMx forums.")}
@@ -159,9 +164,10 @@ mxMI <- function(model, matrices=NA, full=TRUE){
 			ret <- c(ret, mxMI(asubmodel)) #probably won't work.
 		}
 	}
-	return(list(MI=mi.r, MI.Full=mi.f, plusOneParamModels=new.models, EPC=epc.f))
+	if(is.null(mi.f)) mi.f <- rep(NA, length(mi.r))
+	if(is.null(epc.f)) epc.f <- rep(NA, length(mi.r))
+	retList <- list2DF(list(MI=mi.r, MI.Full=mi.f, plusOneParamModels=new.models, EPC=epc.f), nrow=length(mi.r))
+	return(retList)
 }
 
 
-
-
