@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2020 by the individuals mentioned in the source code history
+#   Copyright 2007-2021 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -350,13 +350,13 @@ mxPath <- function(from, to = NA,
     step <- rep(NA_integer_, length(arrows))
     step[arrows==0] <- 1L  # default step
   }
-	if (all.na(to)) { to <- as.character(to) }
-	if (all.na(from)) { from <- as.character(from) }
-	if (all.na(values)) { values <- as.numeric(values) }
-	if (all.na(labels)) { labels <- as.character(labels) }
-	if (all.na(lbound)) { lbound <- as.numeric(lbound) }
-	if (all.na(ubound)) { ubound <- as.numeric(ubound) }
-	if (all.na(connect)) { connect <- as.character(connect) }
+	if (isAllNa(to)) { to <- as.character(to) }
+	if (isAllNa(from)) { from <- as.character(from) }
+	if (isAllNa(values)) { values <- as.numeric(values) }
+	if (isAllNa(labels)) { labels <- as.character(labels) }
+	if (isAllNa(lbound)) { lbound <- as.numeric(lbound) }
+	if (isAllNa(ubound)) { ubound <- as.numeric(ubound) }
+	if (isAllNa(connect)) { connect <- as.character(connect) }
 	pathCheckVector(from, 'from', is.character, 'character')
 	pathCheckVector(to, 'to', is.character, 'character')
 	pathCheckVector(arrows, 'arrows', is.numeric, 'numeric')
@@ -449,3 +449,90 @@ displayPath <- function(object) {
 
 setMethod("print", "MxPath", function(x,...) { displayPath(x) })
 setMethod("show", "MxPath", function(object) { displayPath(object) })
+
+#Maybe export this as an imx* function?:
+AllRAMOrLISREL <- function(model,submodels=TRUE){
+	out <- TRUE
+	if(length(model$expectation)>0){
+		# TODO: Add LISREL expectation once the backend knows how to calculate analytic matrix derivs for it:
+		# out <- out && ( is(model$expectation,"MxExpectationRAM") || is(model$expectation,"MxExpectationLISREL") )
+		out <- out && is(model$expectation,"MxExpectationRAM")
+	}
+	else{
+		return(FALSE) #<--It has neither RAM (nor--TODO--LISREL) expectation, because it has no expectation.
+	}
+	if(submodels && length(model$submodels) > 0){
+		out <- out && all(sapply(model@submodels, AllRAMOrLISREL))
+	}
+	return(out)
+}
+
+#Maybe export this as an imx* function?:
+AnyRAMOrLISREL <- function(model,submodels=TRUE){
+	out <- FALSE
+	if(length(model$expectation)>0){
+		# TODO: Add LISREL expectation once the backend knows how to calculate analytic matrix derivs for it:
+		# out <- out || is(model$expectation,"MxExpectationRAM") || is(model$expectation,"MxExpectationLISREL")
+		out <- out || is(model$expectation,"MxExpectationRAM")
+	}
+	else{
+		if(!submodels){
+			return(FALSE) #<--It has neither RAM (nor--TODO--LISREL) expectation, because it has no expectation.
+		}
+	}
+	if(submodels && length(model$submodels) > 0){
+		out <- out || all(sapply(model@submodels, AnyRAMOrLISREL))
+	}
+	return(out)
+}
+
+##' imxHasAlgebraOnPath
+##'
+##' This is an internal function exported for those people who know
+##' what they are doing.  This function checks if a model (or any of its
+##' submodels) either (1) has labels on MxPaths that reference one or more
+##' MxAlgebras, or (2) defines any of the RAM matrices as MxAlgebras.
+##'
+##' @param model an MxModel object
+##' @param submodels logical; recursion over child models?
+##' @param strict logical; raise error if `model` contains no paths?
+imxHasAlgebraOnPath <- function(model, submodels=TRUE, strict=FALSE){
+	#out <- FALSE
+	if(!AnyRAMOrLISREL(model,submodels=TRUE)){
+		if(strict){
+			#Throw error when strict, because model has no paths:
+			stop(paste0(omxQuotes(model$name)," or one of its submodels does not use MxPaths")) 
+		}
+		else{
+			return(FALSE) #<--Can't have algebras on paths if there are no paths!
+		}
+	}
+	if(length(model@algebras)){
+		allAlgNames <- names(model@algebras)
+		#We need to check to see if the RAM matrices themselves are algebras:
+		if( (model$expectation$A %in% allAlgNames) || (model$expectation$S %in% allAlgNames) || 
+				(model$expectation$F %in% allAlgNames) ){
+			#^^^Admittedly, it is difficult to imagine a scenario in which the user has specified the 'F' matrix as an *algebra*,
+			#but we'd better check just in case.
+			return(TRUE)
+		}
+		if( !is.na(model$expectation$M) && (model$expectation$M %in% allAlgNames) ){
+			return(TRUE)
+		}
+		allPathLabels <- names(omxGetParameters(model,free=F)) #<--Paths that have labels referencing an algebra must be fixed.
+		for(i in 1:length(allAlgNames)){
+			#out <- out || length(grep(paste0(allAlgNames[i],"\\["),allPathLabels))
+			#We are looking for the complete algebra name, followed by an opening square bracket:
+			if(length(grep(paste0(allAlgNames[i],"\\["),allPathLabels))){ 
+				return(TRUE)
+			}
+			# if(out){
+			# 	return(out)
+			# }
+		}
+	}
+	if(submodels && length(model$submodels) > 0){
+		return(any(sapply(model$submodels,imxHasAlgebraOnPath)))
+	}
+	return(FALSE) #<--Function couldn't find any reason to return TRUE.
+}
