@@ -207,8 +207,9 @@ void omxMatrix::disconnect()
 	}
 
 	if(fitFunction) {
-		delete fitFunction;
-		fitFunction = NULL;
+    auto *ff = fitFunction;
+		fitFunction = 0;
+		delete ff;
 	}
 }
 
@@ -532,9 +533,13 @@ void omxMatrix::addPopulate(omxMatrix *from, int srcRow, int srcCol, int destRow
 
 void omxMatrix::unshareMemoryWithR()
 {
-	if (!owner) return;
+	if ( !owner || (rows * cols == 0L) ) return;
+	
+	if(OMX_DEBUG){
+		mxLog("Now unsharing memory for %s, with %d rows and %d columns",name(),rows,cols);
+	}
 
-	double *copy = (double*) R_Realloc(NULL, rows * cols, double);
+	double *copy = (double*) R_Calloc(rows * cols, double);
 	memcpy(copy, data, rows * cols * sizeof(double));
 	setData(copy);
 	owner = NULL;
@@ -898,10 +903,7 @@ void omxMatrixHorizCat(omxMatrix** matList, int numArgs, omxMatrix* result)
 
 	for(int j = 0; j < numArgs; j++) {
 		if(totalRows != matList[j]->rows) {
-			char *errstr = (char*) calloc(250, sizeof(char));
-			sprintf(errstr, "Non-conformable matrices in horizontal concatenation (cbind). First argument has %d rows, and argument #%d has %d rows.", totalRows, j + 1, matList[j]->rows);
-			omxRaiseError(errstr);
-			free(errstr);
+			omxRaiseErrorf("Non-conformable matrices in horizontal concatenation (cbind). First argument has %d rows, and argument #%d has %d rows.", totalRows, j + 1, matList[j]->rows);
 			return;
 		}
 		totalCols += matList[j]->cols;
@@ -947,10 +949,7 @@ void omxMatrixVertCat(omxMatrix** matList, int numArgs, omxMatrix* result)
 
 	for(int j = 0; j < numArgs; j++) {
 		if(totalCols != matList[j]->cols) {
-			char *errstr = (char*) calloc(250, sizeof(char));
-			sprintf(errstr, "Non-conformable matrices in vertical concatenation (rbind). First argument has %d cols, and argument #%d has %d cols.", totalCols, j + 1, matList[j]->cols);
-			omxRaiseError(errstr);
-			free(errstr);
+			omxRaiseErrorf("Non-conformable matrices in vertical concatenation (rbind). First argument has %d cols, and argument #%d has %d cols.", totalCols, j + 1, matList[j]->cols);
 			return;
 		}
 		totalRows += matList[j]->rows;
@@ -1001,10 +1000,7 @@ void omxMatrixTrace(omxMatrix** matList, int numArgs, omxMatrix* result)
         int ncol  = inMat->cols;
 
     	if(nrow != ncol) {
-    		char *errstr = (char*) calloc(250, sizeof(char));
-    		sprintf(errstr, "Non-square matrix in Trace().\n");
-    		omxRaiseError(errstr);
-    		free(errstr);
+    		omxRaiseErrorf("Non-square matrix in Trace().\n");
             return;
     	}
 
